#include "bussim_arc.h"
#include "bussim_global.h"
#include "math.h"
#include <iostream>
#include <stdio.h>
#
#include <GL/gl.h>
#include <freeglut/freeglut.h>

BUSSIM_ARC::BUSSIM_ARC()
{
    this->ID = -1;

    this->orig_node = -1;
    this->dest_node = -1;
    this->orig_screen_x = -1;
    this->orig_screen_y = -1;
    this->dest_screen_x = -1;
    this->dest_screen_y = -1;

    this->length_screen = 0.0;
    this->pos = -1;

    this->first_line = nullptr;
    this->last_line = nullptr;
    this->lines_on_arc = 0;

    this->type = -1;

    this->first_waiting_vehicle = nullptr;
    this->last_waiting_vehicle = nullptr;

}

BUSSIM_ARC::BUSSIM_ARC(int _ID, int _ORIGIN, int _DESTINATION, double _ORIG_SCREEN_X, double _ORIG_SCREEN_Y, double _DEST_SCREEN_X, double _DEST_SCREEN_Y)
{
    this->ID = _ID;
    this->orig_node = _ORIGIN;
    this->dest_node = _DESTINATION;
    this->orig_screen_x = _ORIG_SCREEN_X;
    this->orig_screen_y = _ORIG_SCREEN_Y;
    this->dest_screen_x = _DEST_SCREEN_X;
    this->dest_screen_y = _DEST_SCREEN_Y;
    this->pos = -1;

    this->first_line = nullptr;
    this->last_line = nullptr;
    this->lines_on_arc = 0;
    //this->length = 0.0;
    this->length_screen = 0.0;

    this->type = -1;

    this->first_waiting_vehicle = nullptr;
    this->last_waiting_vehicle = nullptr;

}

BUSSIM_ARC::~BUSSIM_ARC()
{

}

void BUSSIM_ARC::update_length(void)
{
    //this->length = sqrt(pow(this->dest_x -this->orig_x,2)+pow(this->dest_y-this->orig_y,2));
    this->length_screen = sqrt(pow(this->dest_screen_x-this->orig_screen_x,2)+pow(this->dest_screen_y-this->orig_screen_y,2));
}

void BUSSIM_ARC::print_on_screen(void)
{


    // print the line
    glBegin(GL_LINES);
            glLineWidth((GLfloat)30);
            glVertex2f(this->orig_screen_x, this->orig_screen_y);
            glVertex2f(this->dest_screen_x, this->dest_screen_y);
    glEnd();


    // print the arc num
    /*
    glRasterPos2d(this->orig_screen_x + 0.1* (this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 0.1* (this->dest_screen_y-this->orig_screen_y));

    char S[256];
    sprintf(S,"NUM=%d",this->ID);
    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_10, ubuf);
    */

    double diff_x, diff_orth_x;
    double diff_y, diff_orth_y;

    diff_x = this->dest_screen_x - this->orig_screen_x;
    diff_y = this->dest_screen_y - this->orig_screen_y;

    double length_diff;
    length_diff = sqrt(pow(diff_x,2)+pow(diff_y,2));

    diff_orth_x = diff_y;
    diff_orth_y = -diff_x;

    double length_orth;
    length_orth = sqrt(pow(diff_orth_x,2)+pow(diff_orth_y,2));

    glBegin(GL_LINES);
            glVertex2f(this->orig_screen_x + 0.9*(this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 0.9*(this->dest_screen_y-this->orig_screen_y));
            glVertex2f(this->orig_screen_x + 0.9*(this->dest_screen_x-this->orig_screen_x) - 0.01*(this->dest_screen_x-this->orig_screen_x)/length_diff +0.01*diff_orth_x/length_orth, this->orig_screen_y + 0.9*(this->dest_screen_y-this->orig_screen_y)- 0.01*(this->dest_screen_y-this->orig_screen_y)/length_diff+0.01*diff_orth_y/length_orth);
    glEnd();


    glBegin(GL_LINES);
            glVertex2f(this->orig_screen_x + 0.9*(this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 0.9*(this->dest_screen_y-this->orig_screen_y));
            glVertex2f(this->orig_screen_x + 0.9*(this->dest_screen_x-this->orig_screen_x) - 0.01*(this->dest_screen_x-this->orig_screen_x)/length_diff -0.01*diff_orth_x/length_orth, this->orig_screen_y + 0.9*(this->dest_screen_y-this->orig_screen_y)- 0.01*(this->dest_screen_y-this->orig_screen_y)/length_diff-0.01*diff_orth_y/length_orth);
   glEnd();


}

void BUSSIM_ARC::print_queue_on_screen(class BUSSIM_NETWORK *_NET)
{
    int TYPE = -1;
    double x_orth = 0.0;
    double y_orth = 0.0;
    double a = 0.03;

    // determine the middle of the arc
    double x = this->dest_screen_x - this->orig_screen_x;
    double y = this->dest_screen_y - this->orig_screen_y;

    // make necessary calculations
    if( fabs(x) > 0 )
    {
        if( x > 0 )
        {
            if( y > 0 )
            {
                // unten links nach oben rechts (1;1)
                TYPE = 0;
            }
            else
            {
                // oben links nach unten rechts (1;-1)
                TYPE = 1;
            }
        }
        else
        {
            if( y > 0 )
            {
                // unten rechts nach oben links
                TYPE = 2;
            }
            else
            {
                // oben rechts nach unten links
                TYPE = 3;
            }
        }

                if( (TYPE == 0))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = -sqrt( pow(a,2) - pow(x_orth,2) );
                }
                if( (TYPE == 1))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = -sqrt( pow(a,2) - pow(x_orth,2) );
                }
                if( (TYPE == 2))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = sqrt( pow(a,2) - pow(x_orth,2) );
                }
                if( (TYPE == 3))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = sqrt( pow(a,2) - pow(x_orth,2) );
                }
    }
    else
    {

    }
    // print the arc num

    glRasterPos2d(this->orig_screen_x + 0.5*x+x_orth, this->orig_screen_y + 0.5*y+y_orth);

    char S[256] ,T[256];
    sprintf(S,"queue on arc %d:",this->ID);
    class BUSSIM_ARC_WAITING_VEHICLE *Ptr;
    Ptr = this->first_waiting_vehicle;
    while( Ptr != nullptr )
    {
        sprintf(T," %d",Ptr->veh_id);
        strcat(S, T);
        Ptr = Ptr->next;
    }
    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);

}

void BUSSIM_ARC::append_line(class BUSSIM_ARC_IN_LINE *_Ptr)
{

    if( _Ptr != nullptr )
    {
        if( this->lines_on_arc < 1 )
        {
            // add the first line to this arc
            this->first_line = _Ptr;
            this->last_line = _Ptr;
            this->lines_on_arc++;
        }
        else
        {
            // add another line to this arc
            this->last_line->next = _Ptr;
            this->last_line = _Ptr;
            this->lines_on_arc++;
        }
    }
}

void BUSSIM_ARC::set_direction_type(void)
{
    if( this->orig_screen_x < this->dest_screen_x )
        if( this->orig_screen_y < this->dest_screen_y)
            this->type = BUSSIM_NORTHEAST;
        else
            if( this->orig_screen_y > this->dest_screen_y)
                this->type = BUSSIM_SOUTHEAST;
            else
                this->type = BUSSIM_EAST;
    else
        if( this->orig_screen_x > this->dest_screen_x )
            if( this->orig_screen_y < this->dest_screen_y)
                this->type = BUSSIM_NORTHWEST;
            else
                if( this->orig_screen_y > this->dest_screen_y)
                    this->type = BUSSIM_SOUTHWEST;
                else
                    this->type = BUSSIM_WEST;
        else
            if( this->orig_screen_y < this->dest_screen_y)
                this->type = BUSSIM_NORTH;
            else
                this->type = BUSSIM_SOUTH;
}

void BUSSIM_ARC::append_arrived_vehicle(int _VEH_ID)
{
    if( this->last_waiting_vehicle != nullptr )
    {
        // append to existing sequence
        this->last_waiting_vehicle->next = new class BUSSIM_ARC_WAITING_VEHICLE;
        this->last_waiting_vehicle->next->veh_id = _VEH_ID;
        this->last_waiting_vehicle->next->next = nullptr;
        this->last_waiting_vehicle = this->last_waiting_vehicle->next;

    }
    else
    {
        // store the first vehicle into the queue
        this->first_waiting_vehicle = new class BUSSIM_ARC_WAITING_VEHICLE;
        this->first_waiting_vehicle->veh_id = _VEH_ID;
        this->first_waiting_vehicle->next = nullptr;
        this->last_waiting_vehicle = this->first_waiting_vehicle;
    }
}

int BUSSIM_ARC::remove_vehicle_from_queue(int _VEH_ID, class BUSSIM_NETWORK *_NET)
{
    int ret_val = 0;

    //std::cout << "BEGIN REMOVE VEHICLE " << _VEH_ID << " on arc " << this->ID << std::endl;
    if( this->first_waiting_vehicle != nullptr )
    {
        // there is at least one vehicle in the queue
        if( this->first_waiting_vehicle->next == nullptr )
        {
            // there is exactly one vehicle in the queue
            if( this->first_waiting_vehicle->veh_id == _VEH_ID )
            {
                // the one and only saved vehicle must be removed
                delete this->first_waiting_vehicle;
                this->first_waiting_vehicle = nullptr;
                this->last_waiting_vehicle = nullptr;
            }
        }
        else
        {
            // there are two or even more vehicles in the queue

            // first look for the requested vehicle
            class BUSSIM_ARC_WAITING_VEHICLE *Ptr, *Tmp;
            Ptr = this->first_waiting_vehicle;
            if ( Ptr->veh_id == _VEH_ID )
            {
                // the first entry must be removed
                this->first_waiting_vehicle = this->first_waiting_vehicle->next;
                delete Ptr;
            }
            else
            {

                // std::cout << "A+" << std::endl;
                Tmp = this->first_waiting_vehicle;
                // std::cout << "Current Queue" << std::endl;
                /*
                while( Tmp != nullptr )
                {
                    std::cout << Tmp->veh_id << " POS="
                    << _NET->VEHICLE[_VEH_ID].POS
                    << " " << _NET->VEHICLE[_VEH_ID].ON_ARC.ID
                    << " " << _NET->VEHICLE[_VEH_ID].km_travelled_on_arc
                    << " " << _NET->VEHICLE[_VEH_ID].remaing_waiting_time
                    << std::endl;
                    Tmp = Tmp->next;
                }
                std::cout << "LAST=" << this->last_waiting_vehicle->veh_id << std::endl;
                */

                // first identify the associated entry
                Tmp = nullptr;
                while( Ptr != nullptr )
                {
                    if( Ptr->veh_id == _VEH_ID )
                    {
                        Tmp = Ptr;
                        Ptr = nullptr;
                        //delete Tmp;
                    }
                    else
                    {
                        Ptr = Ptr->next;
                    }
                }
                //std::cout << "A" << std::endl;
                // now remove the identified queue entry
                if( Tmp != nullptr )
                {
                    if( Tmp == this->first_waiting_vehicle )
                    {
                        // we have to remove the first vehicle in queue
                        this->first_waiting_vehicle = this->first_waiting_vehicle->next;
                        delete Tmp;
                    }
                    else
                    {
                        ret_val = -1;
                        /*
                        // goto the predecessor vehicle
                        Ptr = this->first_waiting_vehicle;
                        while( Ptr->next != Tmp)
                            Ptr = Ptr->next;

                        if( Tmp == this->last_waiting_vehicle )
                        {
                            // we have to remove the last vehicle in queue
                            Ptr->next = nullptr;
                            this->last_waiting_vehicle = Ptr;
                            delete Tmp;
                        }
                        else
                        {
                            // we have to remove a vehicle in between two other queued vehicles
                            Ptr->next = Ptr->next->next;
                            delete Tmp;
                        }
                        */
                    }
                }
                //std::cout << "A-" << std::endl;
            }
        }
    }
    else
    {
        std::cout << "ERROR" << std::endl;
        exit(0);
    }

    //std::cout << "END REMOVE VEHICLE" << std::endl;

    return ret_val;
}

