#ifndef BUSSIM_ARC_H_INCLUDED
#define BUSSIM_ARC_H_INCLUDED

// the following class template is used to save the sequence of currently waiting vehicles at a node
// added by version 1.05
class BUSSIM_ARC_WAITING_VEHICLE {
public:
    int veh_id;
    class BUSSIM_ARC_WAITING_VEHICLE *next;
};


class BUSSIM_ARC_IN_LINE {
public:
    int LINE_ID;
    double x_offset;
    double y_offset;
    class BUSSIM_ARC_IN_LINE *next;
};

class BUSSIM_ARC {
public:
    int ID;                 // unique identifier
    int orig_node;          // refers to id of source BUSSIM_POINT of this arc
    int dest_node;          // refers to id of target BUSSIM_POINT of this arc

    int pos;                // only needed if arc is saved in line
    double length_screen;   // length on screen (scaled into [-1;1]x[-1;1] determined by Pythagoras

    int type;               // defines in which direction this arc shows

    class BUSSIM_ARC_IN_LINE *first_line;
    class BUSSIM_ARC_IN_LINE *last_line;
    int lines_on_arc;

    double orig_screen_x;   // coordinates on screen
    double orig_screen_y;
    double dest_screen_x;
    double dest_screen_y;

    class BUSSIM_ARC_WAITING_VEHICLE *first_waiting_vehicle;    // added by version 1.05
    class BUSSIM_ARC_WAITING_VEHICLE *last_waiting_vehicle;    // added by version 1.05

    BUSSIM_ARC();
    BUSSIM_ARC(int ID, int _ORIGIN, int _DESTINATION, double _ORIG_SCREEN_X, double _ORIG_SCREEN_Y, double _DEST_SCREEN_X, double _DEST_SCREEN_Y);
    ~BUSSIM_ARC();

    void print_on_screen(void);
    void print_queue_on_screen(class BUSSIM_NETWORK *_NET);
    void update_length(void);
    void append_line(class BUSSIM_ARC_IN_LINE *_Ptr);
    void set_direction_type(void);
    // V1.05: appends the vehicle _VEH_ID to the local waiting queue
    void append_arrived_vehicle(int _VEH_ID);
    // V1.05 remove the first vehicle from the local waiting queue
    int remove_vehicle_from_queue(int _VEH_ID, class BUSSIM_NETWORK *_NET);
};

#endif // BUSSIM_ARC_H_INCLUDED
