#include "bussim_line.h"
#include "bussim_network.h"
#include "bussim_global.h"
#include "math.h"

#include <iostream>
#include <string>
#include <stdio.h>
#include <GL/gl.h>
#include <freeglut/freeglut.h>

using namespace std;

BUSSIM_LINE::BUSSIM_LINE()
{
    this->ID = -1;
    this->SERVICE = -1;
    this->ARCS = 0;
    this->USED_ARC = nullptr;
    this->ID_NEXT_LINE = -1;
    this->x_offset = 0.0;
    this->y_offset = 0.0;
    this->displayed = 1;
    sprintf(this->LineName,"undefined");
    this->type = BUSSIM_REGULARLINE;
}

BUSSIM_LINE::BUSSIM_LINE(int _ID, int _SERVICE, int _ID_NEXT_LINE, double _RED, double _GREEN, double _BLUE, int _LINETYPE)
{
    this->ID = _ID;
    this->SERVICE = _SERVICE;
    this->RED = _RED;
    this->GREEN = _GREEN;
    this->BLUE = _BLUE;
    this->ARCS = 0;
    this->USED_ARC = nullptr;
    this->ID_NEXT_LINE = _ID_NEXT_LINE;
    this->x_offset = 0.0;
    this->y_offset = 0.0;
    this->displayed = 1;
    sprintf(this->LineName,"line %d - service %d",this->ID,this->SERVICE);
    this->type = _LINETYPE;
}

BUSSIM_LINE::~BUSSIM_LINE()
{
    if( this->ARCS > 0 )
        delete[] this->USED_ARC;
}

void BUSSIM_LINE::append_arc(class BUSSIM_ARC _ARC)
{
    if( this->ARCS == 0 )
    {
        // nothing saved so far, open a new array
        this->USED_ARC = new class BUSSIM_ARC[1];
        this->USED_ARC[0] = _ARC;
    }
    else
    {
        class BUSSIM_ARC AUX[this->ARCS];
        for( int i=0 ; i < this->ARCS ; i++ )
            AUX[i] = this->USED_ARC[i];

        delete[] this->USED_ARC;

        this->USED_ARC = new class BUSSIM_ARC[this->ARCS+1];
        for( int i=0 ; i < this->ARCS ; i++ )
            this->USED_ARC[i] = AUX[i];

        this->USED_ARC[this->ARCS] = _ARC;
    }

    // we use pos to find out the position of an arc in a line
    this->USED_ARC[this->ARCS].pos = this->ARCS;

    this->ARCS++;
}

void BUSSIM_LINE::set_offset()
{
    this->x_offset = 0.0025;
    this->y_offset = 0.0025;

    if( this->ARCS > 0 )
    {
        this->x_offset = 0.01 + (double)(this->SERVICE)/500.0;
        this->y_offset = - (0.01  + (double)(this->SERVICE)/500.0);


        /*
            if( (this->USED_ARC[0].orig_screen_x <= this->USED_ARC[this->ARCS-1].dest_screen_x)
               && (this->USED_ARC[0].orig_screen_y <= this->USED_ARC[this->ARCS-1].dest_screen_y) )
               {
                   // from left buttom to right top
                this->x_offset = 0.01 + (double)(this->SERVICE)/200.0;
                this->y_offset = - (0.01  + (double)(this->SERVICE)/200.0);
               }
            if( (this->USED_ARC[0].orig_screen_x >= this->USED_ARC[this->ARCS-1].dest_screen_x)
               && (this->USED_ARC[0].orig_screen_y >= this->USED_ARC[this->ARCS-1].dest_screen_y) )
               {
                   // from left buttom to right top
                this->x_offset = - (0.01 + (double)(this->SERVICE)/200.0);
                this->y_offset = + (0.01  + (double)(this->SERVICE)/200.0);
               }
            if( (this->USED_ARC[0].orig_screen_x <= this->USED_ARC[this->ARCS-1].dest_screen_x)
               && (this->USED_ARC[0].orig_screen_y >= this->USED_ARC[this->ARCS-1].dest_screen_y) )
               {
                   // from left buttom to right top
                this->x_offset = - (0.01 + (double)(this->SERVICE)/200.0);
                this->y_offset = - (0.01  + (double)(this->SERVICE)/200.0);
               }
            if( (this->USED_ARC[0].orig_screen_x >= this->USED_ARC[this->ARCS-1].dest_screen_x)
               && (this->USED_ARC[0].orig_screen_y <= this->USED_ARC[this->ARCS-1].dest_screen_y) )
               {
                   // from left buttom to right top
                this->x_offset = + (0.01 + (double)(this->SERVICE)/200.0);
                this->y_offset = + (0.01  + (double)(this->SERVICE)/200.0);
               }
        */
    }
}
void BUSSIM_LINE::print_on_screen(class BUSSIM_NETWORK *_NET)
{
    int LINES_ON_THIS_ARC = 0;
    class BUSSIM_ARC_IN_LINE *LinePtr;

    double x,y;
    double x_orth, y_orth;
    double a = 0.01;

    //std::cout << "line length=" << this->ARCS << std::endl;
    if( this->displayed > 0 )
    {


        // print the connection
        //glColor4b(this->RED, this->GREEN, this->BLUE, 0.1);
        double r = 0.000;
        int num_segments = 0;
        for(int i=0 ; i < this->ARCS ; i++)
        {
            // print the bubble of the start
            glEnable( GL_LINE_SMOOTH );
            glLineWidth((GLfloat)3.1);
            glBegin(GL_LINE_LOOP);
            //glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
            for(int ii = 0; ii < num_segments; ii++)
            {
                float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

                float x = r * cosf(theta);//calculate the x component
                float y = r * sinf(theta);//calculate the y component

                glVertex2f(this->USED_ARC[i].orig_screen_x+this->x_offset, this->USED_ARC[i].orig_screen_y+this->y_offset);
                glVertex2f(x + this->USED_ARC[i].orig_screen_x+this->x_offset, y + this->USED_ARC[i].orig_screen_y+this->y_offset);//output vertex

            }
            glEnd();
            // print the bubble of the start
            glBegin(GL_LINE_LOOP);
            //glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
            for(int ii = 0; ii < num_segments; ii++)
            {
                float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

                float x = r * cosf(theta);//calculate the x component
                float y = r * sinf(theta);//calculate the y component

                glVertex2f(this->USED_ARC[i].dest_screen_x+this->x_offset, this->USED_ARC[i].dest_screen_y+this->y_offset);
                glVertex2f(x + this->USED_ARC[i].dest_screen_x+this->x_offset, y + this->USED_ARC[i].dest_screen_y+this->y_offset);//output vertex

            }
            glEnd();
            // find out the orientation of the line

            double x_alt, y_alt, x_neu, y_neu;

            x_alt = this->USED_ARC[i].orig_screen_x;
            y_alt = this->USED_ARC[i].orig_screen_y;


            //glLineWidth((GLfloat)30);
            // determine number of lines using this arc
            LINES_ON_THIS_ARC = _NET->ARC[this->USED_ARC[i].ID].lines_on_arc;
            // go to the first line serving this arc
            LinePtr = _NET->ARC[this->USED_ARC[i].ID].first_line;

            x_alt = x_alt; //+ 0.025*LinePtr->x_offset;
            y_alt = y_alt; //+ 0.025*LinePtr->y_offset;
            for(int j=0 ; j < 2*LINES_ON_THIS_ARC ; j++)
            {

                // print the start symbol
            if( i== 0 )
                if( j==0 )
                {
                    x = this->USED_ARC[i].dest_screen_x - this->USED_ARC[i].orig_screen_x;
                    y = this->USED_ARC[i].dest_screen_y - this->USED_ARC[i].orig_screen_y;

                    if( fabs(x) > 0 )
                    {
                        x_orth = sqrt( pow(y,2) / pow(x,2) * pow(a,2) / (1 + pow(y,2) / pow(x,2)) );
                        y_orth = sqrt( pow(a,2) - pow(x_orth,2) );

                        glColor4f(this->RED,this->GREEN,this->BLUE,0.0f);

                        glBegin(GL_LINES);
                        glVertex2f(this->USED_ARC[i].orig_screen_x, this->USED_ARC[i].orig_screen_y);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + x_orth, this->USED_ARC[i].orig_screen_y + y_orth);
                        glEnd();

                        x_orth = -sqrt( pow(y,2) / pow(x,2) * pow(a,2) / (1 + pow(y,2) / pow(x,2)) );
                        y_orth = -sqrt( pow(a,2) - pow(x_orth,2) );

                        glColor4f(this->RED,this->GREEN,this->BLUE,0.0f);

                        glBegin(GL_LINES);
                        glVertex2f(this->USED_ARC[i].orig_screen_x, this->USED_ARC[i].orig_screen_y);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + x_orth, this->USED_ARC[i].orig_screen_y + y_orth);
                        glEnd();
                    }
                    else
                    {
                        x_orth = sqrt( pow(a,2) );
                        y_orth = sqrt( 0 );

                        glColor4f(this->RED,this->GREEN,this->BLUE,0.0f);

                        glBegin(GL_LINES);
                        glVertex2f(this->USED_ARC[i].orig_screen_x, this->USED_ARC[i].orig_screen_y);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + x_orth, this->USED_ARC[i].orig_screen_y + y_orth);
                        glEnd();

                        x_orth = -sqrt( pow(a,2) );
                        y_orth = -sqrt( 0 );

                        glColor4f(this->RED,this->GREEN,this->BLUE,0.0f);

                        glBegin(GL_LINES);
                        glVertex2f(this->USED_ARC[i].orig_screen_x, this->USED_ARC[i].orig_screen_y);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + x_orth, this->USED_ARC[i].orig_screen_y + y_orth);
                        glEnd();
                    }

            }

                    x_neu = x_alt + 1.0/(double)(2*LINES_ON_THIS_ARC)*(this->USED_ARC[i].dest_screen_x-this->USED_ARC[i].orig_screen_x);
                    y_neu = y_alt + 1.0/(double)(2*LINES_ON_THIS_ARC)*(this->USED_ARC[i].dest_screen_y-this->USED_ARC[i].orig_screen_y);


                    //x_neu = x_neu + 0.1*LinePtr->x_offset;
                    //y_neu = y_neu + 0.1*LinePtr->y_offset;

                    glColor4f(_NET->LINE[LinePtr->LINE_ID].RED, _NET->LINE[LinePtr->LINE_ID].GREEN, _NET->LINE[LinePtr->LINE_ID].BLUE, 0.0f);

                    glBegin(GL_LINES);
                        if( x_alt <= x_neu )
                        {
                            if( y_alt <= y_neu )
                            {
                                glVertex2f(x_alt + 0.000, y_alt -0.000);
                                glVertex2f(x_neu + 0.000, y_neu -0.000);
                            }
                            else
                            {
                                glVertex2f(x_alt - 0.000, y_alt -0.000);
                                glVertex2f(x_neu - 0.000, y_neu -0.000);
                            }
                        }
                        else
                        {
                            if( y_alt <= y_neu )
                            {
                                glVertex2f(x_alt - 0.000, y_alt +0.000);
                                glVertex2f(x_neu - 0.000, y_neu +0.000);
                            }
                            else
                            {
                                glVertex2f(x_alt + 0.000, y_alt +0.000);
                                glVertex2f(x_neu + 0.000, y_neu +0.000);
                            }
                        }
                    glEnd();

                    x_alt = x_neu;
                    y_alt = y_neu;

                    GLfloat A;
                    glGetFloatv(GL_LINE_WIDTH,&A);
                    //std::cout << "GLW=" << A << std::endl;

                    // go to the next line serving this arc
                    LinePtr = LinePtr->next;
                    if( LinePtr == nullptr )
                        if( j < LINES_ON_THIS_ARC )
                            LinePtr = _NET->ARC[this->USED_ARC[i].ID].first_line;


                glDisable( GL_LINE_SMOOTH );
            }


            //glBegin(GL_LINES);
            //    glVertex2f(this->USED_ARC[i].orig_screen_x + this->x_offset, this->USED_ARC[i].orig_screen_y + this->y_offset);
            //    glVertex2f(this->USED_ARC[i].dest_screen_x + this->x_offset, this->USED_ARC[i].dest_screen_y + this->y_offset);
            //glEnd();


        }
        //glColor4b(1.0, 0.0, 0.0, 1.0);


    }
}

void BUSSIM_LINE::print_on_screen_indicators(class BUSSIM_NETWORK *_NET)
{
    class BUSSIM_ARC_IN_LINE *LinePtr;

    double x_orth, y_orth, x_diff, y_diff;
    double a = 0.01;
    int TYPE = -1;

    //std::cout << "line length=" << this->ARCS << std::endl;
    if( this->displayed > 0 )
    {
        double x_origin, y_origin, x_dest, y_dest;
        double x, y;
        int LINES_ON_ARC;
        for(int i=0 ; i < this->ARCS ; i++)
        {
            // get the coordinates of the recent arc

            x_origin = this->USED_ARC[i].orig_screen_x;
            y_origin = this->USED_ARC[i].orig_screen_y;
            x_dest = this->USED_ARC[i].dest_screen_x;
            y_dest = this->USED_ARC[i].dest_screen_y;

            // determine the appropriate orthogonal vector
            x = x_dest - x_origin;
            y = y_dest - y_origin;

            // print
            glEnable( GL_LINE_SMOOTH );

            if( fabs(x) > 0 )
            {
                if( x > 0 )
                {
                    if( y > 0 )
                    {
                        // unten links nach oben rechts (1;1)
                        TYPE = 0;
                    }
                    else
                    {
                        // oben links nach unten rechts (1;-1)
                        TYPE = 1;
                    }
                }
                else
                {
                    if( y > 0 )
                    {
                        // unten rechts nach oben links
                        TYPE = 2;
                    }
                    else
                    {
                        // oben rechts nach unten links
                        TYPE = 3;
                    }
                }
                if( (TYPE == 0))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = -sqrt( pow(a,2) - pow(x_orth,2) );
                }
                if( (TYPE == 1))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = -sqrt( pow(a,2) - pow(x_orth,2) );
                }
                if( (TYPE == 2))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = sqrt( pow(a,2) - pow(x_orth,2) );
                }
                if( (TYPE == 3))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = sqrt( pow(a,2) - pow(x_orth,2) );
                }

                // determine the number of lines that use this arc
                LINES_ON_ARC = _NET->ARC[this->USED_ARC[i].ID].lines_on_arc;
                x_diff = 0.5*x;
                y_diff = 0.5*y;

                LinePtr = _NET->ARC[this->USED_ARC[i].ID].first_line;
                for( int LC=0 ; LC < LINES_ON_ARC ; LC++ )
                {
                    // print the parallel line
                    glColor4f(_NET->LINE[LinePtr->LINE_ID].RED,_NET->LINE[LinePtr->LINE_ID].GREEN,_NET->LINE[LinePtr->LINE_ID].BLUE,0.0f);
                    glLineWidth(0.5f);
                    glBegin(GL_LINES);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + x_orth + 0.25*x+(double)(LC)/(double)(LINES_ON_ARC)*x_diff, this->USED_ARC[i].orig_screen_y + y_orth + 0.25*y+(double)(LC)/(double)(LINES_ON_ARC)*y_diff);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + x_orth + 0.25*x+(double)(LC+1)/(double)(LINES_ON_ARC)*x_diff, this->USED_ARC[i].orig_screen_y + y_orth + 0.25*y+(double)(LC+1)/(double)(LINES_ON_ARC)*y_diff);
                    glEnd();
                    // print the arc

                    glBegin(GL_POLYGON);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + x_orth + 0.25*x+(double)(LC)/(double)(LINES_ON_ARC)*x_diff+ 0.8*(double)(1)/(double)(LINES_ON_ARC)*x_diff, this->USED_ARC[i].orig_screen_y + y_orth + 0.25*y+(double)(LC)/(double)(LINES_ON_ARC)*y_diff+0.8*(double)(1)/(double)(LINES_ON_ARC)*y_diff);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + 2*x_orth + 0.25*x+(double)(LC)/(double)(LINES_ON_ARC)*x_diff + 0.8*(double)(1)/(double)(LINES_ON_ARC)*x_diff, this->USED_ARC[i].orig_screen_y + 2*y_orth + 0.25*y+(double)(LC)/(double)(LINES_ON_ARC)*y_diff+0.8*(double)(1)/(double)(LINES_ON_ARC)*y_diff);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + x_orth +0.25*x+(double)(LC)/(double)(LINES_ON_ARC)*x_diff + 1.0*(double)(1)/(double)(LINES_ON_ARC)*x_diff, this->USED_ARC[i].orig_screen_y + 1.0* y_orth+ 0.25*y+(double)(LC)/(double)(LINES_ON_ARC)*y_diff+1.0*(double)(1)/(double)(LINES_ON_ARC)*y_diff);
                    glEnd();

                    LinePtr = LinePtr->next;
                }
            }
            else
                    {
                        x_orth = sqrt( pow(a,2) );
                        y_orth = sqrt( 0 );


                        glColor4f(this->RED,this->GREEN,this->BLUE,0.0f);
                        glLineWidth(1.1f);
                        glBegin(GL_LINES);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + x_orth + 0.25*x, this->USED_ARC[i].orig_screen_y + y_orth + 0.25*y);
                        glVertex2f(this->USED_ARC[i].orig_screen_x + x_orth + 0.75*x, this->USED_ARC[i].orig_screen_y + y_orth + 0.75*y);
                        glEnd();
                    }
            glDisable( GL_LINE_SMOOTH );
        }
    }
}

void BUSSIM_LINE::configure(int _ID, int _SERVICE, double _RED, double _GREEN, double _BLUE, const char _LINENAME[256])
{
    this->ID = _ID;           // id of the line
    this->SERVICE = _SERVICE;    // grouping attribute
    this->RED = _RED;
    this->GREEN = _GREEN;
    this->BLUE = _BLUE;
    sprintf(this->LineName,_LINENAME);
}
