#ifndef BUSSIM_NETWORK_H_INCLUDED
#define BUSSIM_NETWORK_H_INCLUDED

#include "bussim_point.h"
#include "bussim_arc.h"
#include "bussim_line.h"
#include "bussim_vehicle.h"
#include "bussim_event.h"

class BUSSIM_NETWORK {
public:
    int POINTS;             // number of network points
    int BUNCHPOINTS;        // number of saved bunching points
    int ARCS;               // number of network arcs
    int TOIS;               // number of things of interest (added in version 1.05)
    int LINES;              // number of lines in the network
    int VEHICLES;           // number of vehicles in the network
    int ACTIVE_VEHICLES;    // number of active vehicles
    int WAITING_VEHICLES;   // number of waiting vehicles

    long EVENTS;         // number of stored events
    long DUTIES;         // number of duties
    int ROTATIONS;       // number of rotations

    double xmin;        // needed to scale everything into [-1;-1] x [-1;1]
    double xmax;
    double ymin;
    double ymax;

    double x_factor;    // needed to maintain an undistored picture of the network
    double y_factor;

    double SPEEDUP_FACTOR;  // speeding up the simulation in automatic mode
    double CURRENT_TIME;    // the current time stamp
    double TIME_STEP;

    int BETRIEBS_ENDE;      // 0 or 1: if 1 all vehicles will be deactived as soon as they reach the terminus of their current line

    int DETECTED_BUNCHES;   // number of requested takeover manoevres

    class BUSSIM_POINT *PT;     // array of network points
    class BUSSIM_ARC *ARC;      // array of network arcs
    class BUSSIM_TOI *TOI;      // array of network tois    added in version 1.05
    class BUSSIM_LINE *LINE;  // array of lines in the network
    class BUSSIM_VEHICLE *VEHICLE;  // array of vehicles in the network

    class BUSSIM_POINT *BP;     // array of detected bunching points

    class BUSSIM_EVENT *first_event;    // used to store all events detected during the simulation
    class BUSSIM_EVENT *last_event;

    class BUSSIM_BUNCHPOINT *first_bp;       // used to store all events detected during the simulation
    class BUSSIM_BUNCHPOINT *last_bp;

    int **PRED;                 // sequencing of pairs of vehicles

    BUSSIM_NETWORK();   // this is the default constructor
    BUSSIM_NETWORK(int _TOIS, int _POINTS, int _ARCS, int _LINES, int _VEHICLES); // this is an overloaded constructor
    ~BUSSIM_NETWORK();  // this is the default constructor

    void specify(void); // use it to define and prepare the network
    void specify_nodes(void);
    void specify_nodes_longlat(void); // added in version 1.05
    void specify_arcs(void);
    void specify_tois(void);            // added in version 1.05
    void specify_lines(void);
    void specify_vehicles(void);
    void specify_rotations(void);
    void display_active_vehicles(void);
    void start_rotations(void);
    void append_bunch_point(class BUSSIM_BUNCHPOINT *_POINT);
    void print_bunch_points(void);
    void append_event(class BUSSIM_EVENT *_EVENT);
    void print_event_list(void);
    int number_of_active_vehicles(void);
    void center_network(void);

    void merge_lines_arcs(void);

    void set_visibility(void);
    void draw_depots(void);
    void draw_service_arcs(void);
    //void draw_poi(void);

    void append_duty_to_vehicle_rotation(int _VEH_ID, int _TRIP_ID);
    void longlat_to_worldxy(void);      // added in version 1.05
    void draw_tois(void);               // added in version 1.05
};


#endif // BUSSIM_NETWORK_H_INCLUDED
